@@ -31,6 +31,8 @@ def tryint(instr):
     except ValueError:    
         string_int = 0
     return string_int
+
+
 class TMDbNameParser():
     def __init__(self, tmdb_api_key, tmdb_lang, ccfcat_hard=None):
         self.ccfcatHard = ccfcat_hard
@@ -73,7 +75,7 @@ class TMDbNameParser():
         self.poster_path = ''
         self.genre_ids =[]
 
-    def parse(self, torname, TMDb=False, hasIMDb=None):
+    def parse(self, torname, useTMDb=False, hasIMDbId=None):
         self.clearData()
         tc = TorCategory(torname)
         self.ccfcat, self.group = tc.ccfcat, tc.group
@@ -93,9 +95,9 @@ class TMDbNameParser():
 
         self.tmdbcat = transFromCCFCat(self.ccfcat)
 
-        if TMDb:
-            if hasIMDb:
-                tmdbid, title, year = self.searchTMDbByIMDbId(hasIMDb)
+        if useTMDb:
+            if hasIMDbId:
+                tmdbid, title, year = self.searchTMDbByIMDbId(hasIMDbId)
                 if tmdbid > 0:
                     print(self.ccfcat, self.year)
                     return
@@ -105,7 +107,6 @@ class TMDbNameParser():
                                     parseYear, self.cntitle)
             self.ccfcat = transToCCFCat(self.tmdbcat, self.ccfcat)
             
-
     def fixSeasonName(self, seasonStr):
         if re.match(r'^Ep?\d+(-Ep?\d+)?$', seasonStr,
                     flags=re.I) or not seasonStr:
@@ -294,10 +295,12 @@ class TMDbNameParser():
         if title == cntitle:
             cntitle = ''
         cuttitle = re.sub(r'\b(Extended|Anthology|Trilogy|Quadrilogy|Tetralogy|Collections?)\s*$', '', title, flags=re.I)
-        cuttitle = re.sub(r'\b(Extended|HD|S\d+|E\d+|V\d+|4K|DVD|CORRECTED|UnCut)\s*$', '', cuttitle, flags=re.I)
+        cuttitle = re.sub(r'\b(Extended|HD|S\d+|E\d+|V\d+|4K|DVD|CORRECTED|UnCut|SP)\s*$', '', cuttitle, flags=re.I)
         cuttitle = re.sub(r'^\s*(剧集|BBC：?|TLOTR|Jade|Documentary|【[^】]*】)', '', cuttitle, flags=re.I)
         cuttitle = re.sub(r'(\d+部曲|全\d+集.*|原盘|系列|\s[^\s]*压制.*)\s*$', '', cuttitle, flags=re.I)
         cuttitle = re.sub(r'(\b国粤双语|[\b\(]?\w+版|\b\d+集全).*$', '', cuttitle, flags=re.I)
+        cuttitle = re.sub(r'(The[\s\.]*(Complete\w*|Drama\w*|Animate\w*)?[\s\.]*Series|The\s*Movie)\s*$', '', cuttitle, flags=re.I)
+        cuttitle = re.sub(r'\b(Season\s?\d+)\b', '', cuttitle, flags=re.I)
         if cntitle:
             cntitle = re.sub(r'(\d+部曲|全\d+集.*|原盘|系列|\s[^\s]*压制.*)\s*$', '', cntitle, flags=re.I)
             cntitle = re.sub(r'(\b国粤双语|[\b\(]?\w+版|\b\d+集全).*$', '', cntitle, flags=re.I)
@@ -432,7 +435,7 @@ class TMDbNameParser():
         return False
 
     def searchTMDbByIMDbId(self, imdbid):
-        f = Find()
+        f = Find(self.tmdb)
         print("Search : " + imdbid)
         t = f.find_by_imdb_id(imdb_id=imdbid)
         if t:
@@ -450,15 +453,35 @@ class TMDbNameParser():
 
         return self.tmdbid, self.title, self.year
 
-    def searchTMDbByTMDbId(self, cat, tmdbid):
-        if cat == 'tv':
-            tv = TV()
+
+    def searchTMDbByTMDbIdTv(self, tmdbid):
+        tv = TV(self.tmdb)
+        try:
             t = tv.details(tmdbid)
             if t:
                 self.saveTmdbTVResultMatch(t)
-        elif cat == 'move':
-            movie = Movie()
+        except:
+            pass
+        return self.tmdbid, self.title, self.year 
+
+    def searchTMDbByTMDbIdMovie(self, tmdbid):
+        movie = Movie(self.tmdb)
+        try:
             m = movie.details(tmdbid)
             if m:
                 self.saveTmdbMovieResult(m)
+        except:
+            pass
+        return self.tmdbid, self.title, self.year 
+
+    def searchTMDbByTMDbId(self, cat, tmdbid):
+        if cat == 'tv':
+            return self.searchTMDbByTMDbIdTv(tmdbid)
+        elif cat == 'move':
+            return self.searchTMDbByTMDbIdMovie(tmdbid)
+        else:
+            self.searchTMDbByTMDbIdTv(tmdbid)
+            if self.tmdbid <= 0:
+                return self.searchTMDbByTMDbIdMovie(tmdbid)
+
         return self.tmdbid, self.title, self.year 
