@@ -203,7 +203,9 @@ def getSeasonFromFolderName(folderName, failDir=''):
 def fixNtName(file_path):
     # file_path = re.sub(r'(\:|\?|<|>|\*|\\|\")', ' ', file_path)
     if platform.system() == 'Windows':
-        file_path = re.sub(r'(\:|\?|<|>|\*|\\|\")', ' ', file_path)
+        file_path = re.sub(r'(\:|\?|<|>|\*|\\|/|\")', ' ', file_path)
+    else:
+        file_path = re.sub(r'/', ' ', file_path)
     return file_path
 
 
@@ -387,7 +389,7 @@ def fixSeasonGroupWithFilename(folderPath, folderSeason, folderGroup, folderReso
     testFile = getFirstMediaFile(folderPath)
     if testFile:
         p = TMDbNameParser(ARGS.tmdb_api_key, ARGS.tmdb_lang)
-        p.parse(testFile, TMDb=False)
+        p.parse(testFile, useTMDb=False)
         if not folderGroup:
             group = p.group
         if not folderSeason:
@@ -443,9 +445,21 @@ def copyTVFolderItems(tvSourceFolder, genFolder, folderSeason, groupName,
                 # makeLogfile(tvitemPath, seasonFolderFullPath, tvSourceFolder)
                 targetCopy(tvitemPath, seasonFolderFullPath, newTVFileName)
 
+    mkPlexMatch(os.path.join(CATNAME_TV, genFolder), folderTmdbParser)
     targetDirHook(os.path.join(CATNAME_TV, genFolder), tmdbidstr=str(folderTmdbParser.tmdbid))
 
 
+def genMovieTMDbOriginName(mediaSrc, movieName, year, nameParser=None):
+    originName = os.path.basename(mediaSrc)
+    # filename, file_ext = os.path.splitext(mediaSrc)
+    ch1 = ' - '
+    tmdbTail = ''
+    if (nameParser and nameParser.tmdbid > 0 and ARGS.filename_emby_bracket and ARGS.emby_bracket):
+        tmdbTail = ' [tmdbid=' + str(nameParser.tmdbid) + ']'
+    medianame = movieName + ((' (' + year + ')' ) if year else '') + tmdbTail + ch1 + originName
+    return medianame.strip()
+
+
 def genMovieResGroup(mediaSrc, movieName, year, resolution, group, nameParser=None):
     filename, file_ext = os.path.splitext(mediaSrc)
     ch1 = ' - ' if (resolution or group) else ''
@@ -602,14 +616,15 @@ def processMovieDir(mediaSrc, folderCat, folderGenName, folderTmdbParser):
             continue
 
         p = folderTmdbParser
-        fnok = is0DayName(movieItem)
-        if (folderTmdbParser.tmdbid <= 0) or countMediaFiles > 1 or fnok:
-            pf = TMDbNameParser(ARGS.tmdb_api_key, ARGS.tmdb_lang,
-                                ccfcat_hard=setArgsCategory())
-            pf.parse(movieItem, TMDb=(ARGS.tmdb_api_key is not None))
-            pf.title = fixNtName(pf.title)
-            if pf.tmdbid > 0 or fnok:
-                p = pf
+        if (folderTmdbParser.tmdbid <= 0) or countMediaFiles > 1:
+            fnok = is0DayName(movieItem)
+            if fnok:
+                pf = TMDbNameParser(ARGS.tmdb_api_key, ARGS.tmdb_lang,
+                                    ccfcat_hard=setArgsCategory())
+                pf.parse(movieItem, useTMDb=(ARGS.tmdb_api_key is not None))
+                pf.title = fixNtName(pf.title)
+                if pf.tmdbid > 0 or fnok:
+                    p = pf
 
         cat = genCatFolderName(p)
         destFolderName = genMediaFolderName(p)
@@ -627,13 +642,17 @@ def processMovieDir(mediaSrc, folderCat, folderGenName, folderTmdbParser):
             #     copyTVFolderItems(mediaSrc, folderGenName, p.season, p.group,
             #                       p.resolution)
             return
-        elif cat == 'TMDbNotFound':
+        elif cat in ['TMDbNotFound', 'HDTV', 'Audio', 'eBook']:
             targetCopy(mediaSrc, cat)
             targetDirHook(cat, tmdbidstr=str(p.tmdbid))
-            return
+            continue
         else:
             if ARGS.origin_name:
                 newMovieName = os.path.basename(movieItem)
+            elif ARGS.tmdb_origin_name:
+                yearstr = str(p.year) if p.year > 0 else ''
+                newMovieName = genMovieTMDbOriginName(movieItem, p.title, yearstr,
+                                                nameParser=p)
             else:
                 yearstr = str(p.year) if p.year > 0 else ''
                 newMovieName = genMovieResGroup(movieItem, p.title, yearstr,
@@ -644,6 +663,21 @@ def processMovieDir(mediaSrc, folderCat, folderGenName, folderTmdbParser):
             targetDirHook(destCatFolderName, tmdbidstr=str(p.tmdbid))
 
 
+def mkPlexMatch(targetDir, tmdbParser):
+    if not ARGS.make_plex_match:
+        return
+
+    if not tmdbParser:
+        return
+
+    pmfilepath = os.path.join(ARGS.hd_path, targetDir, '.plexmatch')
+    with open(pmfilepath, "w") as pmfile:
+        pmfile.write("Title: %s\ntmdbid: %d\n" %
+                     (tmdbParser.title, tmdbParser.tmdbid))
+        if tmdbParser.year > 1990:
+            pmfile.write("Year: %d\n" % (tmdbParser.year))
+
+
 def targetDirHook(targetDir, tmdbidstr=''):
     # exportTargetDir = os.path.join(ARGS.hd_path, targetDir)
     exportTargetDir = targetDir
@@ -656,7 +690,7 @@ def targetDirHook(targetDir, tmdbidstr=''):
     return
 
 
-def processOneDirItem(cpLocation, itemName):
+def processOneDirItem(cpLocation, itemName, imdbidstr=''):
     global CUR_MEDIA_NAME 
     CUR_MEDIA_NAME = itemName
     mediaSrc = os.path.join(cpLocation, itemName)
@@ -664,11 +698,9 @@ def processOneDirItem(cpLocation, itemName):
         print('\033[31mSKIP symbolic link: [%s]\033[0m ' % mediaSrc)
         return
 
-    imdbidstr = ARGS.imdbid if (ARGS.single and ARGS.imdbid) else ''
-
     cat = setArgsCategory()
     p = TMDbNameParser(ARGS.tmdb_api_key, ARGS.tmdb_lang, ccfcat_hard=cat)
-    p.parse(itemName, TMDb=(ARGS.tmdb_api_key is not None), hasIMDb=imdbidstr)
+    p.parse(itemName, useTMDb=(ARGS.tmdb_api_key is not None), hasIMDbId=imdbidstr)
     p.title = fixNtName(p.title)
     
     cat = genCatFolderName(p)
@@ -689,10 +721,15 @@ def processOneDirItem(cpLocation, itemName):
                 seasonFolderFullPath = os.path.join(ARGS.tv_folder_name, destFolderName,
                                                     p.season)
                 targetCopy(mediaSrc, seasonFolderFullPath, newTVFileName)
+                mkPlexMatch(os.path.join(ARGS.tv_folder_name, destFolderName), p)
                 targetDirHook(os.path.join(ARGS.tv_folder_name, destFolderName), tmdbidstr=str(p.tmdbid))
             elif cat == CATNAME_MOVIE:
                 if ARGS.origin_name:
                     newMovieName = itemName
+                elif ARGS.tmdb_origin_name:
+                    yearstr = str(p.year) if p.year > 0 else ''
+                    newMovieName = genMovieTMDbOriginName(mediaSrc, p.title,
+                                                    yearstr, nameParser=p)
                 else:
                     yearstr = str(p.year) if p.year > 0 else ''
                     newMovieName = genMovieResGroup(mediaSrc, p.title,
@@ -832,6 +869,9 @@ def loadArgs():
     parser.add_argument('--origin-name',
                         action='store_true',
                         help='keep origin file name.')
+    parser.add_argument('--tmdb-origin-name',
+                        action='store_true',
+                        help='filename emby bracket - origin file name.')
     parser.add_argument('--sleep',
                         type=int,
                         help='sleep x seconds after operation.')
@@ -856,12 +896,17 @@ def loadArgs():
     parser.add_argument('--plex-bracket',
                         action='store_true',
                         help='ex: Alone (2020) {tmdb-509635}')
+    parser.add_argument('--make-plex-match',
+                        action='store_true',
+                        help='Create a .plexmatch file at the top level of a series')
     parser.add_argument('--after-copy-script',
                         default='',
                         help='call this script with destination folder path after link/move')
     parser.add_argument('--imdbid',
                         default='',
                         help='specify the TMDb id, -s single mode only')
+    parser.add_argument('--site-str',
+                        help='site-id(ex. hds-12345) folder name, set site strs like (\'chd,hds,ade,ttg\').')
 
     global ARGS
     ARGS = parser.parse_args()
@@ -870,6 +915,69 @@ def loadArgs():
     makeKeepExts()
 
 
+def hasIMDbId(str):
+    m1 = re.search(r'\[imdb(id)?\=(tt\d+)\]', str.strip(), flags=re.A | re.I)
+    m2 = re.search(r'(tt\d+)\s*$', str, flags=re.A | re.I)
+    if m1:
+        return m1[2]
+    elif m2:
+        return m2[1]
+    else:
+        return None
+
+
+def matchSiteId(str):
+    if ARGS.site_str:
+        # argSites = ARGS.site_str.split(',')
+        # siteStr = '|'.joint(argSites)
+        siteStr = ARGS.site_str.replace(',', '|')
+    else:
+        siteStr = 'chd|hds|ourbits|hdc|ttg|ade|cmct|frds|pter|u2|mteam|hdh|lemon'
+    # m1 = re.search(r'(chd|hds|ourbits|hdc|ttg|ade|cmct|frds|pter|u2|mteam|hdh|lemon)-(\d+)$', str.strip(), flags=re.A | re.I)
+    m1 = re.search(r'('+siteStr+')-(\d+)$', str.strip(), flags=re.A | re.I)
+    if m1:
+        return m1[2]
+    else:
+        return None
+
+
+def onlyOneDirInSiteIdFolder(cpLocation, foldername):
+    siteid = matchSiteId(foldername)
+    if siteid:
+        # dirlist = [name for name in os.listdir(os.path.join(cpLocation, foldername)) if os.path.isdir(os.path.join(cpLocation, foldername, name))]
+        dirlist = os.listdir(os.path.join(cpLocation, foldername))
+        if len(dirlist) >= 1:
+            return siteid, dirlist[0]
+    return '', ''
+
+
+def onlyOneDirInIMDbFolder(cpLocation, foldername):
+    imdbstr = hasIMDbId(foldername)
+    if imdbstr:
+        dirlist = os.listdir(os.path.join(cpLocation, foldername))
+        if len(dirlist) >= 1:
+            return imdbstr, dirlist[0]
+        # dirlist = [name for name in os.listdir(os.path.join(cpLocation, foldername)) if os.path.isdir(os.path.join(cpLocation, foldername, name))]
+        # if len(dirlist) == 1:
+            # return imdbstr, dirlist[0]
+    return '', ''
+
+
+def parseFolderIMDbId(locIn, itemIn):
+    siteid, insideSiteFolderName = onlyOneDirInSiteIdFolder(locIn, itemIn)
+    folderIMDb, insideIMDbFolderName = onlyOneDirInIMDbFolder(locIn, itemIn)
+    if folderIMDb:
+        parentLocation = os.path.join(locIn, itemIn)
+        itemName = insideIMDbFolderName
+    elif siteid:
+        parentLocation = os.path.join(locIn, itemIn)
+        itemName = insideSiteFolderName
+    else:
+        parentLocation = locIn
+        itemName = itemIn
+    return parentLocation, itemName, folderIMDb
+
+
 def main():
     loadArgs()
     cpLocation = ARGS.MEDIA_DIR
@@ -882,38 +990,51 @@ def main():
         searchCache = CacheManager(cpLocation)
         searchCache.openCache()
 
+    argIMDb = ARGS.imdbid if (ARGS.single and ARGS.imdbid) else ''
+
     if os.path.isfile(cpLocation):
         processOneDirItem(os.path.dirname(cpLocation),
-                          os.path.basename(os.path.normpath(cpLocation)))
+                          os.path.basename(os.path.normpath(cpLocation)), argIMDb)
     else:
         if ARGS.single and not isCollections(cpLocation):
-            processOneDirItem(os.path.dirname(cpLocation),
+            # processOneDirItem(os.path.dirname(cpLocation),
+            #                   os.path.basename(os.path.normpath(cpLocation)), imdbidstr)
+            
+            parentLocation, itemName, folderimdb = parseFolderIMDbId(os.path.dirname(cpLocation),
                               os.path.basename(os.path.normpath(cpLocation)))
+            if argIMDb:
+                processOneDirItem(parentLocation, itemName, argIMDb)
+            else:
+                processOneDirItem(parentLocation, itemName, folderimdb)
+
         else:
             for torFolderItem in os.listdir(cpLocation):
                 if uselessFile(torFolderItem):
                     continue
-                if isCollections(torFolderItem) and os.path.isdir(
-                        os.path.join(cpLocation, torFolderItem)):
+                
+                parentLocation, itemName, folderimdb = parseFolderIMDbId(cpLocation, torFolderItem)
+
+                if isCollections(itemName) and os.path.isdir(
+                        os.path.join(parentLocation, itemName)):
                     print('\033[35mProcess collections: %s \033[0m' %
-                          torFolderItem)
-                    packDir = os.path.join(cpLocation, torFolderItem)
+                          itemName)
+                    packDir = os.path.join(parentLocation, itemName)
                     for fn in os.listdir(packDir):
                         if ARGS.cache:
                             if searchCache.isCached(fn):
-                                print('\033[32mSkipping. File previously linked: %s \033[0m' % ( fn ))
+                                print('\033[32mSkipping. File previously linked: %s \033[0m' % (fn))
                                 continue
                             else:
+                                processOneDirItem(packDir, fn, imdbidstr='')
                                 searchCache.append(fn)
-                        processOneDirItem(packDir, fn)
                 else:
                     if ARGS.cache:
-                        if searchCache.isCached(torFolderItem):
-                            print('\033[32mSkipping. File previously linked: %s \033[0m' % ( torFolderItem ))
+                        if searchCache.isCached(itemName):
+                            print('\033[32mSkipping. File previously linked: %s \033[0m' % (itemName))
                             continue
                         else:
-                            searchCache.append(torFolderItem)
-                    processOneDirItem(cpLocation, torFolderItem)
+                            processOneDirItem(parentLocation, itemName, folderimdb)
+                            searchCache.append(itemName)
 
     if ARGS.cache:
         searchCache.closeCache()
